c
c     FILE: eplusminus.f                    Creation date: 21/AUG/1996.
c                                       LAST MODIFICATION: 14/APR/2003.
c
c     This file contains the routines which deal with e+ e-
c     processing.
c
c<---><---><---><---><---><---><--->*<---><---><---><---><---><---><--->
c<---><---><---><---><---><---><--->*<---><---><---><---><---><---><--->
c
      subroutine epmadv(npart, pdatarr, nrem, remidx)
c
c     Advancing electron or positrons and deciding their fates.
c
c     Written by: S. J. Sciutto, La Plata 1996, 1997, 1998, 1999;
c                                Fermilab 1999;
c                                La Plata 2000, 2001, 2003.
c
c     Arguments:
c     =========
c
c     npart.......... (input, integer) The number of entries to
c                     process.
c     pdatarr........ (input, character*(*), array(npart))
c                     Array containing the particle data.
c     nrem........... (output, integer) Number of remaining particles
c                     in the stack.
c     remidx......... (output, integer, array(npart)) Indices of
c                     remaining particles.
c
c
      implicit none
c
c     Compilation parameters.
c
      include 'constants.f'
      include 'pclepar.f'
      include 'pclecodes.f'
      include 'modelpar.f'
c
c     Declaration of arguments.
c
      integer           npart, nrem
      character*(*)     pdatarr(npart)
      integer           remidx(npart)
c
c     Particle data storage templates
c
      include 'pdata.f'
c
c     Declaration of shared data.
c
      include 'pclecomm.f'
      include 'modelcomm.f'
c
c     Declaration of internal variables and arrays.
c
      integer           istack, charge
      double precision  inke2, lnvmin, meanbrempathneg
      double precision  f, u, s, v, vv, uvv, wsq, q, ke2
      double precision  path, step, maxpath, totpath
      double precision  alos, clos, blos, egyc, eminloss
      double precision  alosl, closl, blosl, dlosl
      double precision  korate, koratefa, koratefb, kopath
      double precision  lossrate, weff, prevtime
      logical           geosw
      logical           ispositron, nomore, plost, plowe
      double precision  screen
      double precision  fa4, fb4
      double precision  va, vb, vf, qq
      double precision  c, ke1, ct2, st2, cp2, sp2, upsec0(3)
      logical           goon, fatebrem, fatekno, fatecarry
      double precision  urandomt, urandom
c
c     FIRST EXECUTABLE STATEMENT
c
      nrem = 0
c
      eminloss = emineploss
c
c     Particle stack loop.
c
      do istack = 1, npart
c
c       Decoding the particle data.
c
        pclerecord = pdatarr(istack)
c
c       Now the following variables are defined and available:
c
c         prcode          Particle type (code).
c         prenergy        Energy (GeV).
c         prwt            Weight (GE 1).
c         prx             \
c         pry              > Particle coordinates (m).
c         prz             /
c         prvz            Vertical (local) altitude (m).
c         prdepth         Depth (g/cm2).
c         prtime          Current particle time (*).
c         prpx            \
c         prpy             > Direction of the impulse (current).
c         prpz            /
c         prpxpr          \  Direction of the impulse (used for last
c         prpypr           > straight line propagation).
c         prpzpr          /
c         prcdepth        Particle creation depth (g/cm2).
c         prctime         Particle creation time (*).
c         prlhdepth       Depth of last hadronic interaction.
c         prbeta          Particle speed (internal use only).
c         prcal           Current atmospheric layer.
c         prlol           Observing level internal data (do not mo-
c                         dify and transmit for decay products).
c         prlowe          Switch to label low energetic particles.
c
c         psparef         double precision spare positions.
c         psparei         Integer spare positions.
c         psparel         Logical spare positions.
c
c       Particle code, prlol and prcal are integers,
c       prlowe is logical and all other variables are double precision.
c       All the named variables can also be referred as positions of
c       the main particle data array "frecord" replacing the "pr" by
c       "ix", for example: pry is equivalent to frecord(ixy).
c
c       (*) Time is set to zero either at the beginning of the shower
c           or at the fisrt primary interaction.
c
c       In this routine prcode should be either "electroncode" or
c       "positroncode".
c
c       The following spare fields are used:
c
c          psparei(0) is a code for the particle fate.
c          psparel(1) is true when the particle must be processed
c                     in the second stack.
c
c       BEGINNING OF PARTICLE PROCESSING.
c
c       Setting some variables that depend on the sign of the charge.
c
        if (prcode .eq. positroncode) then
c
          ispositron = .true.
          charge     = 1
c
          if (prlowe) then
            plowe = .true.
            goto 1100
          endif
c
          alos       = aplos
          blos       = bplos
          clos       = cplos
          alosl      = aplosl
          blosl      = bplosl
          closl      = cplosl
          dlosl      = dplosl
          koratefa   = koratefpa
          koratefb   = koratefpb
c
        else
c
          ispositron = .false.
          charge     = -1
          alos       = aelos
          blos       = belos
          clos       = celos
          alosl      = aelosl
          blosl      = belosl
          closl      = celosl
          dlosl      = delosl
          koratefa   = koratefea
          koratefb   = koratefeb
c
        endif
c
c       Other particle initializations.
c
        call lgtinit(prcode, charge, pclerecord, frecord, geosw)
c
 1005   continue
c
        fatebrem   = .false.
        fatekno    = .false.
        fatecarry  = .true.
        psparel(1) = .false.
        psparei(0) = 0
c
        if (prenergy .gt. minbremenergy) then
c
          lnvmin          = log(ebremenergycut / prenergy)
          meanbrempathneg = brempath / lnvmin
          path            = meanbrempathneg * log(urandomt(0.d0))
c
c         Fate is bremsstrahlung (first trial).
c
          fatebrem  = .true.
          fatecarry = .false.
c
        else
          path   = 1.d8
          lnvmin = -0.001d0
        endif
c
        korate = koratefa + koratefb / prenergy
c
c       Checking knock-on.
c
        if (korate .gt. 1.d-4) then
c
c         exp1 is to reduce calls to log.
c
          if (exp1 .lt. 0) exp1 = -log(urandomt(0.d0))
          kopath = exp1 / korate
c
          if (kopath .lt. path) then
c
c           Fate is knockon.
c
            path = kopath
            exp1 = - log(urandomt(0.d0))
            q    = 1.d0 / prenergy
            ke2  = 1.d0 / (q + urandom() * (invkomin - q))
c
            fatebrem  = .false.
            fatekno   = .true.
            fatecarry = .false.
c
          else
c
            exp1 = exp1 - path * korate
c
          endif
        endif
c
c       Now the electron/positron is going to be advanced.
c       Notice that our z axis points upwards, and therefore prpz is
c       always negative for descending particles.
c
c       The advancing process is done in repeated steps to account
c       for energy loosing. This imposes "in-process" longitudinal
c       monitoring.
c
        totpath = path
c
 1010   continue
c
c       Processing a fraction of the entire path.
c
        if (prdepth .gt. 200.d0) then
          if (prenergy .le. 1.6d-3) then
            maxpath = 0.16d0
          else if (prenergy .gt. 0.04d0) then
            maxpath = 4.00d0
          else
            maxpath = 0.10d3 * prenergy
          endif
        else if (prdepth .gt. 1.0d0) then
          if (prenergy .le. 2.5d-3) then
            maxpath = 0.0177d0
          else if (prenergy .gt. 0.04d0) then
            maxpath = 0.2828d0
          else
            maxpath = 7.0711d0 * prenergy
          endif
          maxpath = maxpath * sqrt(prdepth)
        else if (prdepth .gt. (4 * xquantum)) then
          maxpath = prdepth / 4
        else
          maxpath = xquantum
        endif
c
        if (totpath .gt. maxpath) then
          path    = maxpath
          totpath = totpath - path
        else
          path    = totpath
          totpath = 0
        endif
c
c       Check on positron annihilation in flight.
c       This is done because cross-section rises quickly as energy
c       falls.
c
        if (ispositron) then
c
c         f is approximate estimation of mean kinetic energy along
c         path. Rough loss rate is ~2 MeV/(g/cm2) for air.
c
          f = max(0.0d0, prenergy - 2.0d-3 * path)
          f = 0.5d0 * (f + prenergy)
c
c         Underestimated mfp for annihilation.
c
          if (f .lt. panne1lim) then
            s = panns10 * f * (f + panns11) / (f + panns12)
          else if (f .le. panne2lim) then
            s = panns20 * (f + panns21)
          else
            s = panns30 * (f + panns31)
          endif
c
          if ((s * urandom()) .lt. path) then
c
c           Do more detailed test for annihilation.
c
c           Rejection factor calculated using a fit to Heitler formula.
c
            u = log(f)
            if (f .le. panne4lim) then
              v = pannv40 + u * (pannv41 + u * pannv42)
            else
              v = pannv50 + u * (pannv51 + u * pannv52) +
     +            pannv53 / (u + pannv54)
            endif
            v = s * exp(v)
c
            if (urandom() .lt. v) then
c
c             Fate is annihilation.
c
              fatebrem   = .false.
              fatekno    = .false.
              fatecarry  = .false.
              psparei(0) = 5
              psparel(1) = .true.
              path       = path * urandomt(2.d-8)
              totpath    = 0
c
            endif
          endif
        endif
c
c       We treat the atmosphere as a continuum (Layers are for
c       calculation purposes only. No physical meaning.)
c       This implies pathinmed being equal to path.
c
        step = 0.5d0 * path
c
c       Saving current time and particle energy.
c
        prevtime = prtime
        inke2    = prenergy + electronmass
c
c       Entering the advance and longitudinal monitoring procedure
c       (1st. half path).
c
        if (prenergy .ge. eminloss) then
          lossrate = alos - blos / (prenergy + clos)
        else
          egyc     = prenergy + closl
          lossrate = alosl + (dlosl / egyc - blosl) / egyc
        endif
c
        call lgtmaster(prcode, step, electronmass, lossrate, epmcut1,
     +                 pclerecord, frecord, prlol, prcal, prlowe,
     +                 nomore, plost, plowe)
c
        if (nomore) goto 1100
c
c       End of advancing sector.
c
c       We now know that the particle is still stacked.
c
        wsq  = (prenergy + electronmass) * inke2
        weff = sqrt(wsq)
c
c       Magnetic field deflection. The deflection is evaluated here,
c       but takes into account the two halves the path is divided in.
c       It is assumed that the difference between the corresponding
c       lengths (in meters) of both halves is small, especially for
c       those particles suffering significant magnetic delflection
c       (low energy ones).
c
        if (geosw) then
          call geobdeflect(prtime - prevtime, weff, frecord(ixpx))
        endif
c
c       Treating Coulomb and multiple scattering.
c
        if ((weff .gt. erough) .and. (path .gt. 0.002d0)) then
          call coulscatter(epmchimin, frecord, path, sqremass,
     +                     wsq, weff)
        endif 
c
c       Entering the advance and longitudinal monitoring procedure
c       (2nd. half path).
c
        if (prenergy .ge. eminloss) then
          lossrate = alos - blos / (prenergy + clos)
        else
          egyc     = prenergy + closl
          lossrate = alosl + (dlosl / egyc - blosl) / egyc
        endif
c
        call lgtmaster(prcode, step, electronmass, lossrate, epmcut1,
     +                 pclerecord, frecord, prlol, prcal, prlowe,
     +                 nomore, plost, plowe)
c
        if (nomore) goto 1100
c
c       End of advancing sector.
c
c       We now know that the particle is still stacked.
c
c       If there remain some path pieces to advance, return to the
c       initial point.
c
        if (totpath .gt. 0) goto 1010
c
c       The complete path has been advanced, and the particle was
c       not lost.
c       Accordingly with the current particle fate we are going
c       to keep the particle in the stack or return to the
c       beginning for further advancing it.
c
        if (fatecarry) goto 1005
c
        if (fatebrem) then
c
c         Current fate is bremsstrahlung.
c
          if (prenergy .le. ebremenergycut) goto 1005
c
          ke2 = prenergy *
     +          ((ebremenergycut / prenergy) ** urandomt(0.d0))
c
c         The preceding approximate distribution produces secondary
c         gammas with energies in the range [ebremenergycut, prenergy].
c
c         Taking into account atomic screening effects. The following
c         code tries to reproduce Rossi's differential cross-sections.
c
          u  = prenergy + electronmass
          v  = ke2 / u
          vv = 1 - v
          va = 1 + vv ** 2
          vb = 0.66666666667d0 * vv
c
          if (vv .gt. 0) then
            uvv    = u * vv
            screen = screenfac * v / uvv
          else
            uvv    = electronmass
            screen = screenlim
          endif
c
          if (screen .lt. 2.d0) then
            fa4 = 5.21d0 - screen * (0.98d0 - 0.158d0 * screen)
            fb4 = fa4 - 0.168d0 * exp(-4 * screen)
            f   = va * (fa4 - thirdlogz) - vb * (fb4 - thirdlogz)
          else
            c   = 1.d0 /
     +            (screen * (2.281d0 + 0.025d0 * screen ** 2))
            f   = 4.798317367d0 - log(screen) - c - thirdlogz
            f   = f * (va - vb)
          endif
c
          if ((urandom() * mefo) .lt. f) then
c
c           Correction for Landau-Pomeranchuk-Migdal effect.
c
            if (prenergy .gt. lpmthreshold) then
              call lpmeffect(ke2, u, u - ke2, prdepth, .false., goon)
            else
              goon = .true.
            endif
c
c           End of processing lpm effect.
c
            if (goon) then
c
c             Ok with bremsstrahlung. (if goon is false no action is
c             taken and the particle remains in the stack for
c             further processing).
c
              ke1 = prenergy - ke2
              c   = 40.d6 * (u ** 2) / log(2 * uvv /
     +                                     (v * electronmass))
              f   = urandom()
              ct2 = 1 + (f - 1) / (0.5d0 + c * f)
c
c             Deflecting and emitting the radiated photon.
c            
              call deflectr(frecord(ixpx), ct2, st2, cp2, sp2, upsec0)
c
              f = pfreeegy(prcode)
              call stack1pclek(1, prenergy + f, ke1 + f,
     +                         gammacode, ke2,
     +                         prwt, frecord(ixx), prdepth, prtime,
     +                         prlhdepth, upsec0, prlol, prcal, goon)
c
c             Advancing a new time only if the particle must continue
c             in the stack.
c
              if (.not. goon) goto 1120
c
c             The particle looses energy.
c
              prenergy = prenergy - ke2
              plowe    = (prenergy .le. epmcut1)
              prlowe   = plowe
              if (plowe) goto 1100
c
            endif
          endif
          goto 1005
c
        else if (fatekno) then
c
c         Current fate is knock-on.
c         Checking the energy of secondary electron, taking into
c         account the current primary energy.
c
          if (ke2 .lt. 2.d-6) goto 1005
c
          if (ispositron) then
c
            if (ke2 .gt. (prenergy - 2.d-6)) goto 1005
c
c           Bhabha cross sections for positrons.
c
            u  = prenergy + electronmass
            q  = ke2 / prenergy
            f  = 1 - q
            qq = f ** 2
            vf = (1 - q * f) ** 2
            if (q .lt. 0.2d0) then
              v = vf
            else
              v = 0.21d-3 + 0.82d-3 * q
              v = 1 - v / (prenergy + 0.875d-3 +
     +                    52.5d-3 * f * (qq ** 2))
              v = v * vf
            endif
c
          else
c
            if ((2 * ke2) .gt. prenergy) goto 1005
c
c           Cross section for electrons.
c
            u  = prenergy + electronmass
            q  = ke2 / prenergy
            f  = 1.d0 / (1.d0 - q)
            qq = (u / prenergy) ** 2
            v  = qq * (f ** 2) + q ** 2
            v  = v - (3 * qq - 1) * q * f
            v  = v * prenergy / (2.25d0 * (u + electronmass))
c
          endif
c
          if (urandom() .lt. v) then
c
c           Knock-on accepted.
c           Ok with knock-on. (otherwise no action is taken and
c           the particle remains in the stack for further processing).
c
            ke1 = prenergy - ke2
            ct2 = sqrt(ke2 * (u + electronmass) /
     +                 (prenergy * (ke2 + twoemass)))
c
c           Deflecting and emitting the secondary electron.
c
            call deflectr(frecord(ixpx), ct2, st2, cp2, sp2, upsec0)
c
            f = pfreeegy(prcode)
            call stack1pclek(1, prenergy + f, ke1 + f,
     +                       electroncode, ke2, prwt,
     +                       frecord(ixx), prdepth,
     +                       prtime, prlhdepth, upsec0,
     +                       prlol, prcal, goon)
c
c           Advancing a new time only if the particle must continue
c           in the stack.
c
            if (.not. goon) goto 1120
c
c           The particle looses energy.
c
            prenergy = ke1
            plowe    = (prenergy .le. epmcut1)
            prlowe   = plowe
            if (plowe) goto 1100
c
          endif
          goto 1005
c
        endif
c
c       The particle's fate is annihilation.
c       Keeping in the stack for further processing.
c
        nrem         = nrem + 1
        remidx(nrem) = istack
c
c       Reencoding the patricle data record.
c
        pdatarr(istack) = pclerecord
c
        goto 1120
c
c       Entry for lost and low energetic particles.
c
 1100   continue
c
c       Low energetic particles are kept in the stack, and
c       if positrons they are forced to annihilation.
c
        if (plowe) then
c
          psparel(1) = (ispositron .and. forceloweannih)
          if (psparel(1)) then
            psparei(0) = 5
            ke1        = prenergy * urandomt(1.d-4)
            call plostegy(prcode, prlol, prwt * (prenergy - ke1))
            prenergy   = ke1
          else
            psparei(0) = 9
          endif
c
c         Keeping in the stack for further processing.
c
          nrem         = nrem + 1
          remidx(nrem) = istack
c
c         Reencoding the patricle data record.
c
          pdatarr(istack) = pclerecord
c
        endif
c
 1120   continue
c
c       End of particle stack loop.
c
      enddo
c
      return
      end
c     --- End of routine epmadv
c
c<---><---><---><---><---><---><--->*<---><---><---><---><---><---><--->
c
      subroutine epmdecay(npart, pdatarr, nrem, remidx)
c
c     Processing the electrons and positrons that have already been
c     advanced by routine epmadv, accordingly with the fates that were
c     set.
c
c     Written by: S. J. Sciutto, La Plata 1996, 1997, 2000.
c
c
c     Arguments:
c     =========
c
c     npart.......... (input, integer) The number of entries to
c                     process.
c     pdatarr........ (input, character*(*), array(npart))
c                     Array containing the particle data.
c     nrem........... (output, integer) Number of remaining particles
c                     in the stack.
c     remidx......... (output, integer, array(npart)) Indices of
c                     remaining particles.
c
c
      implicit none
c
c     Compilation parameters.
c
      include 'constants.f'
      include 'pclecodes.f'
c
c     Declaration of arguments.
c
      integer           npart, nrem
      character*(*)     pdatarr(npart)
      integer           remidx(npart)
c
c     Particle data storage templates
c
      include 'pdata.f'
c
c     Declaration of shared data.
c
      include 'modelcomm.f'
      include 'modelcomm2.f'
c
c     Declaration of internal variables and arrays.
c
      integer           istack
      double precision  u, vv, c, f
      double precision  p, pp, uop, e0, e1
      logical           ktp
      double precision  urandom
c
c     FIRST EXECUTABLE STATEMENT
c
      nrem = 0
c
      do istack = 1, npart
c
c       Decoding the particle data.
c
        pclerecord = pdatarr(istack)
c
c       Now the following variables are defined and available:
c
c         prcode          Particle type (code).
c         prenergy        Energy (GeV).
c         prwt            Weight (GE 1).
c         prx             \
c         pry              > Particle coordinates (m).
c         prz             /
c         prvz            Vertical (local) altitude (m).
c         prdepth         Depth (g/cm2).
c         prtime          Current particle time (*).
c         prpx            \
c         prpy             > Direction of the impulse (current).
c         prpz            /
c         prpxpr          \  Direction of the impulse (used for last
c         prpypr           > straight line propagation).
c         prpzpr          /
c         prcdepth        Particle creation depth (g/cm2).
c         prctime         Particle creation time (*).
c         prlhdepth       Depth of last hadronic interaction.
c         prbeta          Particle speed (internal use only).
c         prcal           Current atmospheric layer.
c         prlol           Observing level internal data (do not mo-
c                         dify and transmit for decay products).
c         prlowe          Switch to label low energetic particles.
c
c         psparef         double precision spare positions.
c         psparei         Integer spare positions.
c         psparel         Logical spare positions.
c
c       Particle code, prlol and prcal are integers,
c       prlowe is logical and all other variables are double precision.
c       All the named variables can also be referred as positions of
c       the main particle data array "frecord" replacing the "pr" by
c       "ix", for example: pry is equivalent to frecord(ixy).
c
c       (*) Time is set to zero either at the beginning of the shower
c           or at the fisrt primary interaction.
c
c       The following spare fields are used:
c
c          psparei(0) is a code for the particle fate.
c          psparel(1) is true when the particle must be processed
c                     in the second stack.
c
c       BEGINNING OF PARTICLE PROCESSING.
c
c       Switching accordigly with the preselected fate.
c
c       Knock on is completely processed in the advance procedure.
c       Bremsstrahlung fate is completely processed in the
c       advance routine, so only annihilation must be
c       processed here.
c
        if (psparel(1)) then
c
c         Processing annihilation.
c
          u  = prenergy + twoemass
          p  = u / electronmass
          c  = p * p - 2
          pp = sqrt(p * (p - 2))
          e0 = 1 / (p + pp)
          vv = log((1 - e0) / e0)
c
c         Acceptance-rejection algorithm to simulate Heitler's formula.
c
 1010     continue
          e1 = e0 * exp(urandom() * vv)
          f  = 1 / e1 + p * (p * e1 - 2)
          if ((c * urandom()) .lt. f) goto 1010
c
c         Evaluating energies and angles of secondary gammas.
c
          esec(1) = e1 * u
          esec(2) = u - esec(1)
          uop     = p / pp
          wa(1)   = (1 - electronmass / esec(1)) * uop
          wa(2)   = (1 - electronmass / esec(2)) * uop
c
c         Deflecting and stacking the gammas.
c
          call pdeflectr(frecord(ixpx), wa, 3, upsec)
c
          seccode(1) = gammacode
          seccode(2) = gammacode
c
          call stacknpcled(1, u, 2, seccode, esec,
     +                     prwt, frecord(ixx), prdepth, prtime,
     +                     prlhdepth, 3, upsec, prlol, prcal,
     +                     0, ktp, prwt, wa)
c
        else
c
c         The particle is kept in the stack.
c
          nrem         = nrem + 1
          remidx(nrem) = istack
c
        endif
c
      enddo
c
      return
c
      end
c     --- End of routine epmdecay
c
c<---><---><---><---><---><---><--->*<---><---><---><---><---><---><--->
c<---><---><---><---><---><---><--->*<---><---><---><---><---><---><--->
c
c     End of file 'eplusminus.f'
c     This source file is part of AIRES 2.8.4a distribution.
c
c     Ay48299427284ay 0 Tue Dec 12 16:29:48 ART 2006
