c
c     FILE: initsim.f                       Creation date: 14/JUL/1997.
c                                       LAST MODIFICATION: 17/MAY/2004.
c
c     This file contains several initialization routines called from
c     the simulation programs.
c
c<---><---><---><---><---><---><--->*<---><---><---><---><---><---><--->
c<---><---><---><---><---><---><--->*<---><---><---><---><---><---><--->
c
      subroutine initialremark
c
c     Inserting (if it corresponds) an initial remark before the
c     user-specified remarks.
c
c     Written by: S. J. Sciutto, La Plata 2000.
c
c
      implicit none
c
c     Compilation parameters.
c
      include 'initrem.f'
      include 'initpar.f'
c
c     Declaration of shared data.
c
      include 'initcomm.f'
c
c     Declaration of internal variables and arrays.
c
      integer           i, ll, sll
c
c     FIRST EXECUTABLE STATEMENT
c
      call strimcopy(inirem, len(inirem), auxline, ll)
      if (ll .gt. 0) then
        sll = min(ll + 1, 75)
        write(auxfilestring, 2010) ('-', i = 1, sll)
 2010   format(75a1)
        call rmkcopy(auxfilestring, sll)
        call rmkcopy(auxline, ll)
        call rmkcopy(auxfilestring, sll)
      endif
c
      return
      end
c     --- End of routine initialremark
c
c<---><---><---><---><---><---><--->*<---><---><---><---><---><---><--->
c
      subroutine init2
c
c     Initializations to be performed before starting the first
c     simulation job.
c
c     Written by: S. J. Sciutto, La Plata 1996, 1997, 1999;
c                                Fermilab 1999;
c                                La Plata 1999, 2000, 2001, 2003.
c
c
      implicit none
c
c     Compilation parameters.
c
      include 'initpar.f'
c     include 'kernelpar.f'      (Included by 'pstackpar.f')
      include 'pstackpar.f'
      include 'pclepar.f'
      include 'ciopar.f'
c
c     Declaration of shared data.
c
      include 'initcomm.f'
      include 'kernelcomm.f'
      include 'pclecomm.f'
      include 'ciocomm.f'
c
c     Declaration of internal variables and arrays.
c
      integer           i, j
      double precision  swt, awt
      double precision  depthfromz
c
c     FIRST EXECUTABLE STATEMENT
c
c     Miscellaneous initializations.
c
      oversionset   = .false.
c
      datistr0(1)   = datistr0(4)
      cpu0(1)       = 0
      cpu0(2)       = 0
      cpu0(3)       = 0
c
      stopchsize1   = 4000
      stopchsize2   = 0
      stopchsize3   = 0
c
      lastshower    = 0
      currshower    = 0
      shcomplete    = .true.
      jobnumber     = 0
      processnumber = 1
c
c     Starting the random number generator.
c
      call raninit(inputrseed)
c
c     Calling the model initialization routine (II).
c
      psta_n(0) = ' '
      call modelinitask(npstacks, psta_n(1), psta_mode,
     +                  2, psta_rn, psta_model_n)
c
c     Normalizing the primary particles weights.
c
      if (nshprimary .gt. 1) then
        swt = 0
        do i = 1, nshprimary
          swt = swt + abs(shprimarywt(i))
        enddo
        if (swt .gt. 0) then
          awt = 0
          do i = 1, nshprimary - 1
            awt = awt + abs(shprimarywt(i)) / swt
            shprimarywt0(i) = awt
          enddo
          shprimarywt0(nshprimary) = 1
        else
          call errprint(0, '*', 4, 'init2',
     +                  'The primary particle weights sum zero!',
     +                  0, 0, nshprimary, shprimarywt, ' ')
        endif
      else
        shprimarywt(1) = 1
      endif
c
c     Evaluating the static shower bounding box parameters.
c     Particles outside the bounding box will be considered "lost"
c     particles.
c
c     Position of top surface.
c
      dtoplost = max(depthfromz(injz + 4.d3, i), 1.d-15)
c
c     Zeroing the main counters and histogram arrays.
c
      call table0
c
c     Initializing the stack statistic arrays and related variables.
c
      do i = 1, npstacks
        avgtotsize(i)     = 0
        peakstsize(1, i)  = 0
        peakstsize(2, i)  = 2100000000
        peakstsize(3, i)  = 0
        procentries(1, i) = 0
        procentries(2, i) = 2100000000d10
        procentries(3, i) = 0
        hardswrite(1, i)  = 0
        hardswrite(2, i)  = 2100000000
        hardswrite(3, i)  = 0
        hardsread(1, i)   = 0
        hardsread(2, i)   = 2100000000
        hardsread(3, i)   = 0
      enddo
c
      do i = 0, npstacks
        callcounter(1, i) = 0
        callcounter(2, i) = 0
      enddo
c
      call statzero(npstacksp1, 5, nplost)
      call statzero(npstacksp1, 5, elost)
      call statzero(npstacksp1, 5, nplowe)
      call statzero(npstacksp1, 5, elowe)
      call statzero(npstacksp1, 5, nprgnd)
      call statzero(npstacksp1, 5, eprgnd)
      call statzero(npstacksp1, 5, eloss)
      call statzero(npstacksp1, 5, totpcles)
c
      call statzero(1, 5, nnotap)
      call statzero(1, 5, enotap)
      call statzero(1, 5, nneutrino)
      call statzero(1, 5, eneutrino)
      call statzero(2, 5, fstintdp)
      call statzero(1, 5, aveprim)
c
c     Special primary statistical variables and related.
c
      do i = 1, nescpcles
        nsprimpart(1, i) = 0
        nsprimpart(2, i) = 2100000000
        nsprimpart(3, i) = 0
      enddo
c
      nintintvar   = 0
      nintfltvar   = 0
      recordspvar0 = .false.
c
c     Setting the first compressed file name number to use.
c
      if (ciosplitevery .gt. 0) then
        j = firstshowernon1 + 1
      else
        j = 0
      endif
c
      do i = 1, npofiles
        pofileno(i) = j
      enddo
c
c     Initializing the cio particle buffers.
c
      call initciopbuf
c
      return
      end
c     --- End of routine init2
c
c<---><---><---><---><---><---><--->*<---><---><---><---><---><---><--->
c
      subroutine init4
c
c     Initializations to be performed before entering the
c     simulation process.
c
c     Written by: S. J. Sciutto, La Plata 1996, 1997, 1998, 1999;
c                                Fermilab 1999;
c                                La Plata 1999, 2000, 2003, 2004.
c
c
      implicit none
c
c     Compilation parameters.
c
      include 'initpar.f'
      include 'kernelpar.f'
      include 'pclepar.f'
      include 'ciopar.f'
      include 'hdatapar.f'
      include 'pclecodes.f'
      include 'constants.f'
c
c     Declaration of shared data.
c
      include 'initcomm.f'
      include 'kernelcomm.f'
      include 'pclecomm.f'
      include 'cioauxcomm.f'
      include 'hdatacomm.f'
c
c     Declaration of internal variables and arrays.
c
      integer           i, j
      double precision  tmp1
      double precision  rmin0depth, rminfilel
      double precision  depthfromz, getinpreal
      logical           getinpswitch
c
c     FIRST EXECUTABLE STATEMENT
c
c     Basic init level 4 stuff.
c
      call init4s
c
c     Setting basic kernel parameters and file names.
c
      nstopfiles = 2
      stopfile(1) = 'AIRES.STOP'
      stopfile(2) = leadingf0(1:leadingf0len) // '.STOP'
      if (wdirnamelen(2) .gt. 0) then
        nstopfiles = 4
        stopfile(3) = wdirname(2)(1:wdirnamelen(2)) // 'AIRES.STOP'
        stopfile(4) = leadingfn(2)(1:leadingfnlen(2)) // '.STOP'
      endif
c
      dstime     = cpu0(4)
      checkcpu   = (cpuperrun .gt. 0)
      checknsh   = (runshowers .gt. 0)
c
      if (processjobs .gt. 0) then
        processruns = processjobs
      else
        processruns = 2095338998
      endif
c
c     Initializing stack variables.
c
      minpstack  = 1
c
c     Setting the histogram pointers.
c
      call tablepointer
c
c     Evaluating some variables needed in the kernel.
c
      tmp1        = depthfromz(injz, atmlayer0)
      groundz00   = rearth + groundz
      injz00      = rearth + injz
      groundz00sq = groundz00 ** 2
      iovgz00sq   = (injz00 / groundz00) ** 2
c
c     varzendis equals 1 or 2. 1 implies that the zenith angle
c     distributes with probability A * sin(Z); 2 with probability
c     B * sin(Z) * cos(Z) = B/2 * sin(2 * Z)
c
      prycoszmin = cos(pryzenithmax * pi180 * varzendis)
      prycoszmax = cos(pryzenithmin * pi180 * varzendis)
      prycosfac  = upi180 / varzendis
c
c     Cut energies.
c
      tmp1 = getinpreal('MesonCutEnergy')
      mincutegy   = tmp1
      do i = -maxpcle, maxpcle
        cutenergy(i, 1) = tmp1
      enddo
c
      nucutenergy = getinpreal('NuclCutEnergy')
      if (nucutenergy .lt. mincutegy) mincutegy = nucutenergy
      do i = lambdacode, maxpcle
        cutenergy(-i, 1) = nucutenergy
        cutenergy( i, 1) = nucutenergy
      enddo
c
      cutenergy(-gammacode, 1) = getinpreal('GammaCutEner')
      cutenergy( gammacode, 1) = cutenergy(-gammacode, 1)
      if (cutenergy(gammacode, 1) .lt. mincutegy)
     +  mincutegy = cutenergy(gammacode, 1)
c
      cutenergy(electroncode, 1) = getinpreal('ElectronCutEner')
      cutenergy(positroncode, 1) = cutenergy(electroncode, 1)
      if (cutenergy(electroncode, 1) .lt. mincutegy)
     +   mincutegy = cutenergy(electroncode, 1)
c
      cutenergy(muminuscode, 1) = getinpreal('MuonCutEner')
      cutenergy(mupluscode, 1) = cutenergy(muminuscode, 1)
      if (cutenergy(muminuscode, 1) .lt. mincutegy)
     +   mincutegy = cutenergy(muminuscode, 1)
c
c     When ForceLowEDecays (ForceLowEAnnihilation) is true, then
c     cutenergy(*, 2) is zero for decaying particles (positrons);
c     Otherwise it is equal to cutenergy(1, *).
c     Actually, the "zero" it is replaced by 1/2 eV to avoid eventual
c     problems with some divisions.
c
      do i = -maxpcle, maxpcle
        cutenergy(i, 2) = cutenergy(i, 1)
      enddo
c
      call getinpstring('ForceLowEDecays', auxline, j)
c
      forcelowedecay = .false.
      if (auxline(1:j) .eq. 'Always') then
        forcelowedecay = .true.
        do i = -maxpcle, maxpcle
          if ((pclelife(i) .ge. 0) .and. (pclelife(i) .lt. 0.5d0))
     +    then
            cutenergy(i, 2) = 5.0d-10
          endif
        enddo
      else if (auxline(1:j) .eq. 'Normal') then
        do i = -maxpcle, maxpcle
          if ((pclelife(i) .ge. 0) .and. (pclelife(i) .lt. 0.5d0) .and.
     +        (cutenergy(i, 1) .lt. pclemass(i)))
     +    then
            cutenergy(i, 2) = 5.0d-10
            forcelowedecay = .true.
          endif
        enddo
      endif
c
      call getinpstring('ForceLowEAnnihilation', auxline, j)
c
      forceloweannih = .false.
      if (auxline(1:j) .eq. 'Always') then
        forceloweannih             = .true.
        cutenergy(positroncode, 2) = 5.0d-10
      else if (auxline(1:j) .eq. 'Normal') then
        if ((cutenergy(gammacode, 1) .lt. 1.2d-3) .and.
     +      (cutenergy(positroncode, 1) .lt. 1.4d-3))  then
          forceloweannih             = .true.
          cutenergy(positroncode, 2) = 5.0d-10
        endif
      endif
c
c     Thinning stuff.
c
      call thininit4
c
c     Geomagnetic field related variables.
c
      tmp1    = - pi180 * geobi
      igbx    = geob * cos(tmp1)
      igbz    = geob * sin(tmp1)
c
      if (geobfluc .ge. 0) then
        igbfluc = geobfluc
      else
        igbfluc = abs(geobfluc) * geob
      endif
c
c     Compressed output files related data.
c
      ciofsplit = (ciosplitevery .gt. 0)
c
      do i = 1, nciofiles
        j = 14 + 2 * i
        rminfile(i)  = fidata(0, j)
        rmaxfile(i)  = fidata(0, j + 1) 
        r2minfile(i) = rminfile(i) ** 2
        r2maxfile(i) = rmaxfile(i) ** 2
      enddo
c
      prsratio0  = getinpreal('ResamplingRatio')
      prssqratio = 1 / (prsratio0 ** 2)
c
      rmin0depth = 0.8d0 * injdepth + 0.2d0 * groundepth
      rminfilel  = rminfile(longiciofile)
c
      do i = 1, nobslevels
        if (obslevdepth(i) .lt. rmin0depth) then
          olrminfile(i, 1) = 0
          totobslev        = i
        else if (obslevdepth(i) .le. groundepth) then
          olrminfile(i, 1) = (obslevdepth(i) - rmin0depth) *
     +                    (rminfilel / (groundepth - rmin0depth))
          totobslev        = i
        else
          olrminfile(i, 1) = rminfilel
        endif
        olrminfile(i, 0) = olrminfile(i, 1) / prsratio0
        olrminfile(i, 2) = olrminfile(i, 1) ** 2
      enddo
c
c     Miscellaneous.
c
      notproprim    = (.not. getinpswitch('PropagatePrimary'))
      resetclock    = (.not. getinpswitch('SetTimeAtInjecti'))
      resetdtoplost = (.not. getinpswitch('SetTopAtInjectio'))
      recordspp     = getinpswitch('RecordSpecPrimaries')
c
c     Calling the model initialization routine (III).
c
      call modelinirun
c
      return
      end
c     --- End of routine init4
c
c<---><---><---><---><---><---><--->*<---><---><---><---><---><---><--->
c
      subroutine newprocess
c
c     Updating the process variables.
c
c     Written by: S. J. Sciutto, La Plata 1996.
c
c
      implicit none
c
c     Compilation parameters.
c
      include 'initpar.f'
c
c     Declaration of shared data.
c
      include 'initcomm.f'
c
c     FIRST EXECUTABLE STATEMENT
c
c     Updating the process counter.
c
      processnumber  = processnumber + 1
c
      return
      end
c     --- End of routine newprocess
c
c<---><---><---><---><---><---><--->*<---><---><---><---><---><---><--->
c
      subroutine newrun
c
c     Updating the run (job) variables.
c
c     Written by: S. J. Sciutto, La Plata 1996.
c
c
      implicit none
c
c     Compilation parameters.
c
      include 'initpar.f'
      include 'kernelpar.f'
c
c     Declaration of shared data.
c
      include 'initcomm.f'
      include 'kernelcomm.f'
c
c     Declaration of internal variables and arrays.
c
      double precision  dummy
c
c     FIRST EXECUTABLE STATEMENT
c
c     Updating the run variables.
c
      jobnumber  = jobnumber + 1
c
      dummy = cpu0(2) + cpu0(3)
      if (dummy .gt. 0) then
        stopchsize1 = (stopchsize2 + stopchsize3) *
     +                           (stopchsecs / dummy) + 1
      endif
c
      return
      end
c     --- End of routine newrun
c
c<---><---><---><---><---><---><--->*<---><---><---><---><---><---><--->
c<---><---><---><---><---><---><--->*<---><---><---><---><---><---><--->
c
c     End of file 'initsim.f'
c     This source file is part of AIRES 2.8.4a distribution.
c
c     Ay48299427284ay 0 Tue Dec 12 16:29:48 ART 2006
